@@ -22,8 +22,9 @@ sys.path.insert(0, os.path.abspath('../..'))
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = [
     'sphinx.ext.autodoc',
+    'oslo_config.sphinxconfiggen',
     # 'sphinx.ext.intersphinx',
-    'oslosphinx'
+    'openstackdocstheme'
 ]
 
 # autodoc generation is a bit aggressive and a nuisance when doing heavy
@@ -33,12 +34,20 @@ extensions = [
 # The suffix of source filenames.
 source_suffix = '.rst'
 
+config_generator_config_file = '../../etc/os-win-config-generator.conf'
+sample_config_basename = '_static/os-win'
+
 # The master toctree document.
 master_doc = 'index'
 
 # General information about the project.
 project = 'os-win'
 copyright = '2015, Cloudbase Solutions Srl'
+# openstackdocstheme options
+openstackdocs_repo_name = 'openstack/os-win'
+openstackdocs_auto_name = False
+openstackdocs_bug_project = 'os-win'
+openstackdocs_bug_tag = ''
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 add_function_parentheses = True
@@ -48,15 +57,14 @@ add_function_parentheses = True
 add_module_names = True
 
 # The name of the Pygments (syntax highlighting) style to use.
-pygments_style = 'sphinx'
+pygments_style = 'native'
 
 # -- Options for HTML output --------------------------------------------------
 
 # The theme to use for HTML and HTML Help pages.  Major themes that come with
 # Sphinx are currently 'default' and 'sphinxdoc'.
 # html_theme_path = ["."]
-# html_theme = '_theme'
-# html_static_path = ['static']
+html_theme = 'openstackdocs'
 
 # Output file base name for HTML help builder.
 htmlhelp_basename = '%sdoc' % project
