@@ -17,6 +17,11 @@
 Hyper-V / Windows related constants.
 """
 
+CIM_STATE_UNKNOWN = 0
+CIM_STATE_OTHER = 1
+CIM_STATE_ENABLED = 2
+CIM_STATE_DISABLED = 3
+
 HYPERV_VM_STATE_OTHER = 1
 HYPERV_VM_STATE_ENABLED = 2
 HYPERV_VM_STATE_DISABLED = 3
@@ -161,3 +166,81 @@ DNS_ZONE_TRANSFER_ALLOWED_ANY_HOST = 0
 DNS_ZONE_TRANSFER_ALLOWED_NAME_SERVERS = 1
 DNS_ZONE_TRANSFER_ALLOWED_SECONDARY_SERVERS = 2
 DNS_ZONE_TRANSFER_NOT_ALLOWED = 3
+
+CLUSTER_GROUP_STATE_UNKNOWN = -1
+CLUSTER_GROUP_ONLINE = 0
+CLUSTER_GROUP_OFFLINE = 1
+CLUSTER_GROUP_FAILED = 2
+CLUSTER_GROUP_PARTIAL_ONLINE = 3
+CLUSTER_GROUP_PENDING = 4
+
+EXPORT_CONFIG_SNAPSHOTS_ALL = 0
+EXPORT_CONFIG_NO_SNAPSHOTS = 1
+EXPORT_CONFIG_ONE_SNAPSHOT = 2
+
+# ACE inheritance flags
+ACE_OBJECT_INHERIT = 0x1
+ACE_CONTAINER_INHERIT = 0x2
+ACE_NO_PROPAGATE_INHERIT = 0x4
+ACE_INHERIT_ONLY = 0x8
+ACE_INHERITED = 0x10
+
+# ACE access masks
+ACE_GENERIC_READ = 0x80000000
+ACE_GENERIC_WRITE = 0x40000000
+ACE_GENERIC_EXECUTE = 0x20000000
+ACE_GENERIC_ALL = 0x10000000
+
+# ACE access modes
+ACE_NOT_USED_ACCESS = 0
+ACE_GRANT_ACCESS = 1
+ACE_SET_ACCESS = 2
+ACE_DENY_ACCESS = 3
+ACE_REVOKE_ACCESS = 4
+ACE_SET_AUDIT_SUCCESS = 5
+ACE_SET_AUDIT_FAILURE = 6
+
+# VLAN operation modes
+VLAN_MODE_ACCESS = 1
+VLAN_MODE_TRUNK = 2
+
+# Action that Hyper-V takes on the VM
+# when the host is shut down.
+HOST_SHUTDOWN_ACTION_TURN_OFF = 2
+HOST_SHUTDOWN_ACTION_SAVE = 3
+HOST_SHUTDOWN_ACTION_SHUTDOWN = 4
+
+# VM snapshot types
+VM_SNAPSHOT_TYPE_DISABLED = 2
+VM_SNAPSHOT_TYPE_PROD_FALLBACK = 3
+VM_SNAPSHOT_TYPE_PROD_ENFORCED = 4
+VM_SNAPSHOT_TYPE_STANDARD = 5
+
+VM_VERSION_5_0 = '5.0'
+VM_VERSION_6_2 = '6.2'
+VM_VERSION_8_0 = '8.0'
+VM_VERSION_254_0 = '254.0'
+
+DEFAULT_WMI_EVENT_TIMEOUT_MS = 2000
+
+SCSI_UID_SCSI_NAME_STRING = 8
+SCSI_UID_FCPH_NAME = 3
+SCSI_UID_EUI64 = 2
+SCSI_UID_VENDOR_ID = 1
+SCSI_UID_VENDOR_SPECIFIC = 0
+
+# The following SCSI Unique ID formats are accepted by Windows, in this
+# specific order of precedence.
+SUPPORTED_SCSI_UID_FORMATS = [
+    SCSI_UID_SCSI_NAME_STRING,
+    SCSI_UID_FCPH_NAME,
+    SCSI_UID_EUI64,
+    SCSI_UID_VENDOR_ID,
+    SCSI_UID_VENDOR_SPECIFIC
+]
+
+DISK_POLICY_UNKNOWN = 0
+DISK_POLICY_ONLINE_ALL = 1
+DISK_POLICY_OFFLINE_SHARED = 2
+DISK_POLICY_OFFLINE_ALL = 3
+DISK_POLICY_OFFLINE_INTERNAL = 4
