@@ -17,32 +17,32 @@ import platform
 
 from oslo_log import log as logging
 
-from os_win._i18n import _, _LE
+from os_win._i18n import _
 from os_win import exceptions
 from os_win.utils import _wqlutils
-from os_win.utils import baseutils
+from os_win.utils.compute import migrationutils
 from os_win.utils.compute import vmutils
-from os_win.utils import jobutils
-from os_win.utils.storage.initiator import iscsi_wmi_utils
 
 LOG = logging.getLogger(__name__)
 
 
-class LiveMigrationUtils(baseutils.BaseUtilsVirt):
+class LiveMigrationUtils(migrationutils.MigrationUtils):
     _STORAGE_ALLOC_SETTING_DATA_CLASS = 'Msvm_StorageAllocationSettingData'
     _CIM_RES_ALLOC_SETTING_DATA_CLASS = 'CIM_ResourceAllocationSettingData'
 
+    _MIGRATION_TYPE_VIRTUAL_SYSTEM = 32768
+    _MIGRATION_TYPE_VIRTUAL_SYSTEM_AND_STORAGE = 32771
+    _MIGRATION_TYPE_STAGED = 32770
+
     def __init__(self):
         super(LiveMigrationUtils, self).__init__()
-        self._vmutils = vmutils.VMUtils()
-        self._jobutils = jobutils.JobUtils()
-        self._iscsi_initiator = iscsi_wmi_utils.ISCSIInitiatorWMIUtils()
 
     def _get_conn_v2(self, host='localhost'):
         try:
-            return self._get_wmi_obj(self._wmi_namespace % host)
+            return self._get_wmi_obj(self._wmi_namespace % host,
+                                     compatibility_mode=True)
         except exceptions.x_wmi as ex:
-            LOG.exception(_LE('Get version 2 connection error'))
+            LOG.exception('Get version 2 connection error')
             if ex.com_error.hresult == -2147217394:
                 msg = (_('Live migration is not supported on target host "%s"')
                        % host)
@@ -50,7 +50,7 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
                 msg = (_('Target live migration host "%s" is unreachable')
                        % host)
             else:
-                msg = _('Live migration failed: %s') % ex.message
+                msg = _('Live migration failed: %r') % ex
             raise exceptions.HyperVException(msg)
 
     def check_live_migration_config(self):
@@ -76,27 +76,11 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
                                              % vm_name)
         return vms[0]
 
-    def _destroy_planned_vm(self, planned_vm):
-        LOG.debug("Destroying existing planned VM: %s",
-                  planned_vm.ElementName)
-        (job_path,
-         ret_val) = self._vs_man_svc.DestroySystem(planned_vm.path_())
-        self._jobutils.check_ret_val(ret_val, job_path)
-
-    def _get_planned_vms(self, conn_v2, vm_name):
-        return conn_v2.Msvm_PlannedComputerSystem(ElementName=vm_name)
-
-    def destroy_existing_planned_vms(self, vm_name):
-        planned_vms = self._get_planned_vms(self._compat_conn, vm_name)
-        for planned_vm in planned_vms:
-            self._destroy_planned_vm(planned_vm)
-
     def _create_planned_vm(self, conn_v2_local, conn_v2_remote,
                            vm, ip_addr_list, dest_host):
         # Staged
-        vsmsd = conn_v2_remote.query("select * from "
-                                     "Msvm_VirtualSystemMigrationSettingData "
-                                     "where MigrationType = 32770")[0]
+        vsmsd = conn_v2_remote.Msvm_VirtualSystemMigrationSettingData(
+            MigrationType=self._MIGRATION_TYPE_STAGED)[0]
         vsmsd.DestinationIPAddressList = ip_addr_list
         migration_setting_data = vsmsd.GetText_(1)
 
@@ -110,42 +94,6 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
 
         return conn_v2_local.Msvm_PlannedComputerSystem(Name=vm.Name)[0]
 
-    def _get_physical_disk_paths(self, vm_name):
-        # TODO(claudiub): Remove this after the livemigrationutils usage has
-        # been updated to create planned VM on the destination host beforehand.
-        ide_ctrl_path = self._vmutils.get_vm_ide_controller(vm_name, 0)
-        if ide_ctrl_path:
-            ide_paths = self._vmutils.get_controller_volume_paths(
-                ide_ctrl_path)
-        else:
-            ide_paths = {}
-
-        scsi_ctrl_path = self._vmutils.get_vm_scsi_controller(vm_name)
-        scsi_paths = self._vmutils.get_controller_volume_paths(scsi_ctrl_path)
-
-        return dict(list(ide_paths.items()) + list(scsi_paths.items()))
-
-    def _get_remote_disk_data(self, vmutils_remote, disk_paths, dest_host):
-        # TODO(claudiub): Remove this after the livemigrationutils usage has
-        # been updated to create planned VM on the destination host beforehand.
-        remote_iscsi_initiator = iscsi_wmi_utils.ISCSIInitiatorWMIUtils(
-            dest_host)
-
-        disk_paths_remote = {}
-        for (rasd_rel_path, disk_path) in disk_paths.items():
-            target = self._iscsi_initiator.get_target_from_disk_path(disk_path)
-            if target:
-                (target_iqn, target_lun) = target
-                dev_num = remote_iscsi_initiator.get_device_number_for_target(
-                    target_iqn, target_lun)
-                disk_path_remote = (
-                    vmutils_remote.get_mounted_disk_by_drive_number(dev_num))
-                disk_paths_remote[rasd_rel_path] = disk_path_remote
-            else:
-                LOG.debug("Could not retrieve iSCSI target "
-                          "from disk path: %s", disk_path)
-        return disk_paths_remote
-
     def _get_disk_data(self, vm_name, vmutils_remote, disk_path_mapping):
         disk_paths = {}
         phys_disk_resources = vmutils_remote.get_vm_disks(vm_name)[1]
@@ -200,11 +148,10 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
         return new_resource_setting_data
 
     def _live_migrate_vm(self, conn_v2_local, vm, planned_vm, rmt_ip_addr_list,
-                         new_resource_setting_data, dest_host):
+                         new_resource_setting_data, dest_host, migration_type):
         # VirtualSystemAndStorage
-        vsmsd = conn_v2_local.query("select * from "
-                                    "Msvm_VirtualSystemMigrationSettingData "
-                                    "where MigrationType = 32771")[0]
+        vsmsd = conn_v2_local.Msvm_VirtualSystemMigrationSettingData(
+            MigrationType=migration_type)[0]
         vsmsd.DestinationIPAddressList = rmt_ip_addr_list
         if planned_vm:
             vsmsd.DestinationPlannedVirtualSystemId = planned_vm.Name
@@ -226,7 +173,7 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
         migr_svc_rmt = conn_v2.Msvm_VirtualSystemMigrationService()[0]
         return migr_svc_rmt.MigrationServiceListenerIPAddressList
 
-    def live_migrate_vm(self, vm_name, dest_host):
+    def live_migrate_vm(self, vm_name, dest_host, migrate_disks=True):
         self.check_live_migration_config()
 
         conn_v2_remote = self._get_conn_v2(dest_host)
@@ -236,37 +183,18 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
         rmt_ip_addr_list = self._get_ip_address_list(conn_v2_remote,
                                                      dest_host)
 
-        planned_vms = self._get_planned_vms(conn_v2_remote, vm_name)
-        if len(planned_vms) > 1:
-            err_msg = _("Multiple planned VMs were found for VM %(vm_name)s "
-                        "on host %(dest_host)s")
-            raise exceptions.OSWinException(
-                err_msg % dict(vm_name=vm_name,
-                               dest_host=dest_host))
-        elif not planned_vms:
-            # TODO(claudiub): Remove this branch after the livemigrationutils
-            # usage has been updated to create planned VM on the destination
-            # host beforehand.
-            planned_vm = None
-            disk_paths = self._get_physical_disk_paths(vm_name)
-            if disk_paths:
-                vmutils_remote = vmutils.VMUtils(dest_host)
-                disk_paths_remote = self._get_remote_disk_data(vmutils_remote,
-                                                               disk_paths,
-                                                               dest_host)
-                planned_vm = self._create_planned_vm(conn_v2_remote,
-                                                     self._compat_conn,
-                                                     vm, rmt_ip_addr_list,
-                                                     dest_host)
-                self._update_planned_vm_disk_resources(
-                    conn_v2_remote, planned_vm, vm_name, disk_paths_remote)
+        planned_vm = self._get_planned_vm(vm_name, conn_v2_remote)
+
+        if migrate_disks:
+            new_resource_setting_data = self._get_vhd_setting_data(vm)
+            migration_type = self._MIGRATION_TYPE_VIRTUAL_SYSTEM_AND_STORAGE
         else:
-            planned_vm = planned_vms[0]
+            new_resource_setting_data = None
+            migration_type = self._MIGRATION_TYPE_VIRTUAL_SYSTEM
 
-        new_resource_setting_data = self._get_vhd_setting_data(vm)
         self._live_migrate_vm(self._compat_conn, vm, planned_vm,
                               rmt_ip_addr_list, new_resource_setting_data,
-                              dest_host)
+                              dest_host, migration_type)
 
     def create_planned_vm(self, vm_name, src_host, disk_path_mapping):
         # This is run on the destination host.
@@ -277,7 +205,7 @@ class LiveMigrationUtils(baseutils.BaseUtilsVirt):
         vm = self._get_vm(conn_v2_remote, vm_name)
 
         # Make sure there are no planned VMs already.
-        self.destroy_existing_planned_vms(vm_name)
+        self.destroy_existing_planned_vm(vm_name)
 
         ip_addr_list = self._get_ip_address_list(self._compat_conn,
                                                  dest_host)
