@@ -23,6 +23,7 @@ from os_win._i18n import _
 from os_win import constants
 from os_win import exceptions
 from os_win.utils.io import ioutils
+from os_win.utils.winapi import constants as w_const
 
 threading = patcher.original('threading')
 time = patcher.original('time')
@@ -31,7 +32,7 @@ LOG = logging.getLogger(__name__)
 
 
 class NamedPipeHandler(object):
-    """Handles asyncronous I/O operations on a specified named pipe."""
+    """Handles asynchronous I/O operations on a specified named pipe."""
 
     _MAX_LOG_ROTATE_RETRIES = 5
 
@@ -65,7 +66,7 @@ class NamedPipeHandler(object):
 
             for job in jobs:
                 worker = threading.Thread(target=job)
-                worker.setDaemon(True)
+                worker.daemon = True
                 worker.start()
                 self._workers.append(worker)
         except Exception as err:
@@ -136,10 +137,10 @@ class NamedPipeHandler(object):
 
         self._pipe_handle = self._ioutils.open(
             self._pipe_name,
-            desired_access=(ioutils.GENERIC_READ | ioutils.GENERIC_WRITE),
-            share_mode=(ioutils.FILE_SHARE_READ | ioutils.FILE_SHARE_WRITE),
-            creation_disposition=ioutils.OPEN_EXISTING,
-            flags_and_attributes=ioutils.FILE_FLAG_OVERLAPPED)
+            desired_access=(w_const.GENERIC_READ | w_const.GENERIC_WRITE),
+            share_mode=(w_const.FILE_SHARE_READ | w_const.FILE_SHARE_WRITE),
+            creation_disposition=w_const.OPEN_EXISTING,
+            flags_and_attributes=w_const.FILE_FLAG_OVERLAPPED)
 
     def _close_pipe(self):
         if self._pipe_handle:
@@ -149,7 +150,7 @@ class NamedPipeHandler(object):
     def _cancel_io(self):
         if self._pipe_handle:
             # We ignore invalid handle errors. Even if the pipe is closed
-            # and the handle reused, by specifing the overlapped structures
+            # and the handle reused, by specifying the overlapped structures
             # we ensure that we don't cancel IO operations other than the
             # ones that we care about.
             self._ioutils.cancel_io(self._pipe_handle, self._r_overlapped,
@@ -220,6 +221,13 @@ class NamedPipeHandler(object):
         except Exception:
             self._stopped.set()
 
+    def flush_log_file(self):
+        try:
+            self._log_file_handle.flush()
+        except (AttributeError, ValueError):
+            # We'll ignore errors caused by closed handles.
+            pass
+
     def _rotate_logs(self):
         self._log_file_handle.flush()
         self._log_file_handle.close()
